@@ -1,10 +1,11 @@
 /*--------------------------------------------------------------------------
-  ----- File:        parseAFM.h
+  ----- File:        parseAFM.c
   ----- Author:      Adobe Systems Inc., modifications by
                      Rainer Menzner (rmz@neuroinformatik.ruhr-uni-bochum.de)
-  ----- Date:        06/03/1998
+  ----- Date:        1999-08-16
   ----- Description: This file is part of the t1-library. It is the original
-                     parseAFM.h modified at a few points.
+                     parseAFM.h modified at a few points, especially for
+		     reading MSDOS-style AFM files..
   ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-1998. 
                      As of version 0.5, t1lib is distributed under the
 		     GNU General Public Library Lincense. The
@@ -80,7 +81,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <sys/file.h>
+#include <sys/types.h>  /* this seems to be necessary on some SCO-systems */
+#if !defined(_MSC_VER)
+#  include <sys/file.h>
+#endif
 #include <math.h>
 #include <string.h>
 #include "parseAFM.h"
@@ -97,6 +101,20 @@
 
 
 
+/* Note: The functions token and linetoken are  extended to be able
+   to parse AFM files with DOS-style line-ends under UNIX. I don't know
+   why AFM files which according to Adobe are explicitly intended to
+   be used under UNIX have MSDOS-style line ends.
+
+   However, we parse the tokens based on the following atoms. This should
+   work on any system.
+*/
+#define CR       '\r'    /* a carriage return */
+#define LF       '\n'    /* a linefeed, which is a newline under UNIX */
+#define CTRL_Z   0x1A    /* some AFM files have this characters as an end
+			    of file indicator. Who know why :) ? */
+
+
 /*************************** GLOBALS ***********************/
 
 static char *ident = NULL; /* storage buffer for keywords */
@@ -117,7 +135,7 @@ static char *ident = NULL; /* storage buffer for keywords */
  * "recognize" procedure to calculate how many possible keys there are.
  */
 
-static enum parseKey {
+typedef enum {
   ASCENDER, CHARBBOX, CODE, COMPCHAR, CAPHEIGHT, COMMENT, 
   DESCENDER, ENCODINGSCHEME, ENDCHARMETRICS, ENDCOMPOSITES, 
   ENDFONTMETRICS, ENDKERNDATA, ENDKERNPAIRS, ENDTRACKKERN, 
@@ -127,7 +145,8 @@ static enum parseKey {
   STARTFONTMETRICS, STARTKERNDATA, STARTKERNPAIRS, 
   STARTTRACKKERN, TRACKKERN, UNDERLINEPOSITION, 
   UNDERLINETHICKNESS, VERSION, XYWIDTH, XWIDTH, WEIGHT, XHEIGHT,
-  NOPE };
+  NOPE} PARSEKEY;
+
 
 /* keywords for the system:  
  * This a table of all of the current strings that are vaild AFM keys.
@@ -163,24 +182,24 @@ static char *keyStrings[] = {
  *  the `linetoken' routine, which does a similar thing but 
  *  reads all tokens until the next end-of-line.
  */
- 
-static char *token(stream)
+static char *token(stream) 
   FILE *stream;
 {
     int ch, idx;
 
-    /* skip over white space */
-    while ((ch = fgetc(stream)) == ' ' || ch == lineterm || 
+    /* skip over white space, instead of the systems EOL-character we check
+       explicitly for CR and LF as well as for ^Z. */
+    while ((ch = fgetc(stream)) == ' ' || ch == CR || ch == LF || ch == CTRL_Z || 
 	   ch == ',' || ch == '\t' || ch == ';');
     
     idx = 0;
     
-    while (ch != EOF && ch != ' ' && ch != lineterm 
-           && ch != '\t' && ch != ':' && ch != ';'){
+    while (ch != EOF && ch != ' ' && ch != CR  && ch != LF &&
+	   ch != CTRL_Z && ch != '\t' && ch != ':' && ch != ';'){
       ident[idx++] = ch;
       ch = fgetc(stream);
     } /* while */
-
+    
     if (ch == EOF && idx < 1) {
       return ((char *)NULL);
     }
@@ -212,7 +231,7 @@ static char *linetoken(stream)
     while ((ch = fgetc(stream)) == ' ' || ch == '\t' ); 
     
     idx = 0;
-    while (ch != EOF && ch != lineterm) 
+    while (ch != EOF && ch != CR  && ch != LF && ch != CTRL_Z) 
     {
         ident[idx++] = ch;
         ch = fgetc(stream);
@@ -236,10 +255,10 @@ static char *linetoken(stream)
  *  The algorithm is a standard Knuth binary search.
  */
 
-static enum parseKey recognize(ident)
+static PARSEKEY recognize(ident)
   register char *ident;
 {
-    int lower = 0, upper = (int) NOPE, midpoint, cmpvalue;
+    int lower = 0, upper = (int) NOPE, midpoint=NOPE, cmpvalue;
     BOOL found = T1LIB_FALSE;
 
     while ((upper >= lower) && !found)
@@ -252,7 +271,7 @@ static enum parseKey recognize(ident)
         else lower = midpoint + 1;
     } /* while */
 
-    if (found) return (enum parseKey) midpoint;
+    if (found) return (PARSEKEY) midpoint;
     else return NOPE;
     
 } /* recognize */
@@ -463,8 +482,7 @@ static int initializeArray(fp, cwi)
         keyword = token(fp);
         if (keyword == NULL)
         {
-	  printf("Loc2\n");
-            error = earlyEOF;
+	    error = earlyEOF;
             break; 
         }
         switch(recognize(keyword))
@@ -548,7 +566,6 @@ static int parseCharWidths(fp, cwi)
           /* Set flag and stop parsing */
         if (keyword == NULL)
         {
-	  printf("Loc3\n");
             error = earlyEOF;
             break; /* get out of loop */
         }
@@ -649,7 +666,6 @@ static int parseCharMetrics(fp, fi)
         keyword = token(fp);
         if (keyword == NULL)
         {
-	  printf("Loc4\n");
             error = earlyEOF;
             break; /* get out of loop */
         }
@@ -761,7 +777,6 @@ static int parseTrackKernData(fp, fi)
         
         if (keyword == NULL)
         {
-	  printf("Loc5\n");
             error = earlyEOF;
             break; /* get out of loop */
         }
@@ -853,110 +868,109 @@ static int parseTrackKernData(fp, fi)
  */
  
 static int parsePairKernData(fp, fi)
-  FILE *fp;
-  register FontInfo *fi;
+     FILE *fp;
+     register FontInfo *fi;
 {  
-    BOOL cont = T1LIB_TRUE, save = (fi->pkd != NULL);
-    int pos = 0, error = ok, pcount = 0;
-    register char *keyword;
+  BOOL cont = T1LIB_TRUE, save = (fi->pkd != NULL);
+  int pos = 0, error = ok, pcount = 0;
+  register char *keyword;
   
-    while (cont)
+  while (cont)
     {
-        keyword = token(fp);
-        
-        if (keyword == NULL)
+      keyword = token(fp);
+      
+      if (keyword == NULL)
         {
-	  printf("Loc6\n");
-            error = earlyEOF;
-            break; /* get out of loop */
+	  error = earlyEOF;
+	  break; /* get out of loop */
         }
-        if (!save)
-          /* get tokens until the end of the Pair Kerning Data */
-          /* section without saving any of the data */
-            switch(recognize(keyword))
-            {
-                case ENDKERNPAIRS:
-                case ENDKERNDATA:
-                    cont = T1LIB_FALSE;
-                    break;
-                case ENDFONTMETRICS:
-                    cont = T1LIB_FALSE;
-                    error = normalEOF;
-                    break;
-                default:
-                    break;
-            } /* switch */
-	else
-          /* otherwise parse entire Pair Kerning Data section, */
-          /* saving the data */
-            switch(recognize(keyword))
-            {
-                case COMMENT:
-                    keyword = linetoken(fp);
-                    break;
-                case KERNPAIR:
-                    if (pcount < fi->numOfPairs)
-                    {
-                        keyword = token(fp);
-                        fi->pkd[pos].name1 = (char *) 
-                            malloc(strlen(keyword) + 1);
-                        strcpy(fi->pkd[pos].name1, keyword);
-                        keyword = token(fp);
-                        fi->pkd[pos].name2 = (char *) 
-                            malloc(strlen(keyword) + 1);
-                        strcpy(fi->pkd[pos].name2, keyword);
-                        keyword = token(fp);
-                        fi->pkd[pos].xamt = atoi(keyword);
-                        keyword = token(fp);
-                        fi->pkd[pos++].yamt = atoi(keyword);
-                        pcount++;
-                    }
-                    else
-                    {
-                        error = parseError;
-                        cont = T1LIB_FALSE;
-                    }
-                    break;
-                case KERNPAIRXAMT:
-                    if (pcount < fi->numOfPairs)
-                    {
-                        keyword = token(fp);
-                        fi->pkd[pos].name1 = (char *) 
-                            malloc(strlen(keyword) + 1);
-                        strcpy(fi->pkd[pos].name1, keyword);
-                        keyword = token(fp);
-                        fi->pkd[pos].name2 = (char *) 
-                            malloc(strlen(keyword) + 1);
-                        strcpy(fi->pkd[pos].name2, keyword);
-                        keyword = token(fp);
-                        fi->pkd[pos++].xamt = atoi(keyword);
-                        pcount++;
-                    }
-                    else
-                    {
-                        error = parseError;
-                        cont = T1LIB_FALSE;
-                    }
-                    break;
-                case ENDKERNPAIRS:
-                case ENDKERNDATA:
-                    cont = T1LIB_FALSE;
-                    break;
-                case ENDFONTMETRICS:
-                    cont = T1LIB_FALSE;
-                    error = normalEOF;
-                    break;
-                case NOPE:
-                default:
-                    error = parseError;
-                    break;
-            } /* switch */
+      if (!save)
+	/* get tokens until the end of the Pair Kerning Data */
+	/* section without saving any of the data */
+	switch(recognize(keyword))
+	  {
+	  case ENDKERNPAIRS:
+	  case ENDKERNDATA:
+	    cont = T1LIB_FALSE;
+	    break;
+	  case ENDFONTMETRICS:
+	    cont = T1LIB_FALSE;
+	    error = normalEOF;
+	    break;
+	  default:
+	    break;
+	  } /* switch */
+      else
+	/* otherwise parse entire Pair Kerning Data section, */
+	/* saving the data */
+	switch(recognize(keyword))
+	  {
+	  case COMMENT:
+	    keyword = linetoken(fp);
+	    break;
+	  case KERNPAIR:
+	    if (pcount < fi->numOfPairs)
+	      {
+		keyword = token(fp);
+		fi->pkd[pos].name1 = (char *) 
+		  malloc(strlen(keyword) + 1);
+		strcpy(fi->pkd[pos].name1, keyword);
+		keyword = token(fp);
+		fi->pkd[pos].name2 = (char *) 
+		  malloc(strlen(keyword) + 1);
+		strcpy(fi->pkd[pos].name2, keyword);
+		keyword = token(fp);
+		fi->pkd[pos].xamt = atoi(keyword);
+		keyword = token(fp);
+		fi->pkd[pos++].yamt = atoi(keyword);
+		pcount++;
+	      }
+	    else
+	      {
+		error = parseError;
+		cont = T1LIB_FALSE;
+	      }
+	    break;
+	  case KERNPAIRXAMT:
+	    if (pcount < fi->numOfPairs)
+	      {
+		keyword = token(fp);
+		fi->pkd[pos].name1 = (char *) 
+		  malloc(strlen(keyword) + 1);
+		strcpy(fi->pkd[pos].name1, keyword);
+		keyword = token(fp);
+		fi->pkd[pos].name2 = (char *) 
+		  malloc(strlen(keyword) + 1);
+		strcpy(fi->pkd[pos].name2, keyword);
+		keyword = token(fp);
+		fi->pkd[pos++].xamt = atoi(keyword);
+		pcount++;
+	      }
+	    else
+	      {
+		error = parseError;
+		cont = T1LIB_FALSE;
+	      }
+	    break;
+	  case ENDKERNPAIRS:
+	  case ENDKERNDATA:
+	    cont = T1LIB_FALSE;
+	    break;
+	  case ENDFONTMETRICS:
+	    cont = T1LIB_FALSE;
+	    error = normalEOF;
+	    break;
+	  case NOPE:
+	  default:
+	    error = parseError;
+	    break;
+	  } /* switch */
     } /* while */
     
-    if (error == ok && pcount != fi->numOfPairs)
-        error = parseError;
+  if (error == ok && pcount != fi->numOfPairs)
+    error = parseError;
         
-    return(error);
+  return(error);
     
 } /* parsePairKernData */    
 
@@ -994,7 +1008,6 @@ static int parseCompCharData(fp, fi)
           /* Have reached an early and unexpected EOF. */
           /* Set flag and stop parsing */
         {
-	  printf("Loc7\n");
             error = earlyEOF;
             break; /* get out of loop */
         }
@@ -1197,7 +1210,6 @@ int T1lib_parseFile (fp, fi, flags)
           /* Have reached an early and unexpected EOF. */
           /* Set flag and stop parsing */
         {
-	  printf("Loc8\n");
             code = earlyEOF;
             break; /* get out of loop */
         }
