@@ -1,10 +1,10 @@
 /*--------------------------------------------------------------------------
   ----- File:        t1misc.h
   ----- Author:      Rainer Menzner (rmz@neuroinformatik.ruhr-uni-bochum.de)
-  ----- Date:        1999-09-02
+  ----- Date:        2000-03-29
   ----- Description: This file is part of the t1-library. It contains
                      some miscellaneous definitions.
-  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-1999. 
+  ----- Copyright:   t1lib is copyrighted (c) Rainer Menzner, 1996-2000. 
                      As of version 0.5, t1lib is distributed under the
 		     GNU General Public Library Lincense. The
 		     conditions can be found in the files LICENSE and
@@ -67,11 +67,19 @@
 #define T1_CONFIGFILENAME    ".t1librc"
 #define ENV_CONF_STRING      "T1LIB_CONFIG"
 #define GLOBAL_CONFIG_FILE   "t1lib.config"
-#define DIRECTORY_SEP        "/"
-#define DIRECTORY_SEP_CHAR   '/'
+#if defined(VMS)
+# define DIRECTORY_SEP        "]"
+# define DIRECTORY_SEP_CHAR   ']'
+#elif defined(_MSC_VER)
+# define DIRECTORY_SEP        "\\"
+# define DIRECTORY_SEP_CHAR   '\\'
+#else
+# define DIRECTORY_SEP        "/"
+# define DIRECTORY_SEP_CHAR   '/'
+#endif
 /* For EMX-environment, we define the default path separator to
    be ';' instead of ':' */
-#ifdef __EMX__
+#if defined(__EMX__) || defined(VMS) || defined(_MSC_VER)
 #define PATH_SEP             ";"
 #define PATH_SEP_CHAR        ';'
 #else
@@ -170,7 +178,6 @@
 #define USES_STANDARD_ENCODING    (short)0x0008
 #define FONT_NOCACHING            (short)0x0100
 
-extern int errno;
 
 #ifndef PI
 #define PI 3.1415927
